import lang from './lang';
const encode = encodeURIComponent;
const decode = (qs) => decodeURIComponent(qs).replace(/\+/g, ' ');

/**
 * Convert a collection to a query string
 * @param {Object|Map|Array} query
 * @return {String}
 */
const stringify = (query) => {
  let entries = Array.isArray(query) ? query : lang.entries(query);
  let serialize = (entry) => entry.slice(0, 2).map(encode).join('=');

  return entries.map(serialize).join('&');
};

/**
 * Explode a query string on its boundaries and strip falsy values
 * @param {String} qs
 * @param {String} delimiter
 * @return {Array}
 */
const tokenize = (qs, boundary='&') => qs.split(boundary).filter(Boolean);

const parse = {};

/**
 * Convert a query string to a list of entries:
 * @example
 *   parse.entries({foo: 1, bar: 2})
 *   >>> [['foo', '1'], ['bar', '2']]
 * @param {String} qs
 * @return {Array}
 */
parse.entries = (qs) => {
  // strip leading question mark
  if (qs && qs[0] === '?') {
    qs = qs.slice(1);
  }

  return tokenize(qs).reduce((out, pair) => {
    out.push(pair.split('=').slice(0, 2).map(decode));
    return out;
  }, []);
};

/**
 * Convert a query string to a Map
 * @param {String} qs
 * @return {Map}
 */
parse.map = (qs) => new Map(parse.entries(qs));

/**
 * Convert a query string to a plain object
 * @param {String} qs
 * @return {Object}
 */
parse.object = (qs) => {
  let reducer = (out, entry) => {
    let [k, v] = entry;
    out[k] = v;
    return out;
  };

  return lang.reduce(parse.entries(qs), reducer, {});
};

export default {
  stringify,
  parse,
};
