import {Tree} from '../../src/tree';

describe('tree', () => {
  describe('Tree', () => {

    describe('#path()', () => {
      it('resolves a path expression', () => {
        let tree = new Tree({data: fixture('tree-1')});

        expect(tree.path('nfc/west/0/name')).to.equal('49ers');
        expect(tree.path('nfc/west/1/name')).to.equal('Seahawks');
      });

      it('handles a leading slash', () => {
        let tree = new Tree({data: fixture('tree-1')});

        expect(tree.path('/nfc/west/0/name')).to.equal('49ers');
        expect(tree.path('/nfc/west/1/name')).to.equal('Seahawks');
      });

      it('caches resolved expressions', () => {
        let cache = {has: stub(), get: stub(), set: stub()};
        let expr = 'nfc/west/0/name';

        let tree = new Tree({
          data: fixture('tree-1'),
          cache: cache,
        });

        cache.has.withArgs(expr).onCall(0).returns(false);
        cache.has.withArgs(expr).onCall(1).returns(true);
        cache.get.withArgs(expr).returns('49ers');

        tree.path(expr);

        expect(cache.has.callCount).to.equal(1);
        expect(cache.get.callCount).to.equal(0);
        expect(cache.set.callCount).to.equal(1);

        tree.path(expr);

        expect(cache.has.callCount).to.equal(2);
        expect(cache.get.callCount).to.equal(1);
        expect(cache.set.callCount).to.equal(1);
      });
    })

    describe('#constructor()', () => {
      it('accepts a tokenizer function', () => {
        let tree = new Tree({
          data: fixture('tree-1'),
          delimiter: ':',
          tokenize: (expr, delimiter) => expr.split(delimiter),
        });

        expect(tree.path('nfc:west:0:name')).to.equal('49ers');
      });
    });

  });
});

